//---------------------------------------------------------------------------
// JrDebug demo that demonstrates various features
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// System includes
#include <iostream>
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Include the Debugging Library files
//  Note that only our main .cpp needs to include the jrdebug_main.cpp file
//  alternatively, you could add this cpp to your project or makefile.
#include "../../jrdebug_main.cpp"
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
int main(int argc, char *argv[])
{
	// automatically parse commandline options and exit if appropriate
	if (JrDebugParseCommandlineArgc("Demo7",&argc,argv,"-db"))
		return 0;

	// main function report
	debugblock("MyMain");

	// here is a loop but with activity push/pop - this will report timings of activities
	// note we enclose it in a {} to properly scope the debugblock
	{
	debugblock("All looping");
	int count3,count4,count5;
	for (int count2=0;count2<10;++count2)
		{
		// report this loop block
		debugblock("Inner loop");
		// take some time
		for (count3=0;count3<30000;++count3)
			{
			for (count4=0;count4<300;++count4)
				{
				// just eat up some cpu cycles
				count5=count3-count2; count5++; count5--; count5=count3-count2;
				}
			}
		}
	}

	// we can also just report current offset time of nearest enclosing debugblock (main)
	debugblockreport("Now for part 2");

	// here is another loop but with activity push/pop - this will report timings of activities
	debugpush("Looping");
	int count3,count4,count5;
	for (int count2=0;count2<10;++count2)
		{
		debugpush("Iterate");
		// take some time
		for (count3=0;count3<3000;++count3)
			{
			// now here is a manual push pop, and further more, we use a * in front of description to avoid display (only used if other messages displayed inside)
			//JrDebug::PushActivity("*innerloop");
			for (count4=0;count4<3000;++count4)
				{
				// just eat up some cpu cycles
				count5=count3-count2; count5++; count5--; count5=count3-count2;
				}
			//JrDebug::PopActivity("*innerloop");
			}
		debugpop("Iterate");
		}
	debugpop("Looping");

	// purposefull failure test
	debugout<<"You should see an error now, to demonstrate unsyncrhonizes push/pop";
	debugpush("ProcedureA");
	debugpop("ProcedureB");

	// anything here is outside of scope of the "all looping" debugblock
	dbprintf("Accomplished testing.");

	// all done
	return 0;
}
//---------------------------------------------------------------------------

