//---------------------------------------------------------------------------
// jrdebug.h
// this is the main header file, which will get included BEFORE all others
//  and whether jrdebug is compiled-in or not.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#define JrDebugVersionString "1.11.01"
#define JrDebugVersionDateString "Nov 14, 2010"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// header guards
#ifndef INC_jrdebugh
#define INC_jrdebugh
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// FIRST SET ANY COMPILER-SPECIFIC SYMBOLS
#include "jrdebug_compilerspecifics.h"
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// NOW ANY SWITCHES:
#ifndef JrDebugNoSwitchfile
 #include "jrdebug_switch.h"
#endif
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
#ifdef JrDebugDIRECTIVE_UseNameSpaces
 // namespaces if configured
 namespace jrdebug {
#endif
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// TYPES
// message identification types - needed even if not compiling in
enum JrdType { JrdMessage, JrdWarning, JrdError, JrdAlarm, JrdNote , JrdTest , JrdCustom , JrdAssertPass, JrdAssertFail, JrdException , JrdTestPass, JrdTestFail, JrdBlockInfo, JrdBlockStart, JrdBlockEnd, JrdBlockEndError, JrdBlockError, JrdInternalInfo, JrdInternalError};
// test-related  message types
enum JrdTestMsgType { JrdTestMsgContinue , JrdTestMsgStop };
// dummy enum to help disambiguate function calls (see jrdebug_core.h)
enum JrEnumDummyFunctionDisambiguator_Assert { JrEnumDummyFunctionDisambiguator_Assert_DummyVal };
// in case we want to use different string classes in the future
// ATTN: can we avoid including this when compiling is disabled
#include <string>
typedef std::string jrd_stdstringT;
typedef void(*JrdFuncpVoidRetT)();
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// For avoiding unused arg warnings - we want to write code that will get compiled out
//#define JrdUnusedArg(Var) (void)(true ? 0 : (void*)&Var)
#define JrdUnusedArg(Var) ((void) (&Var))
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Shortcut Aliases
//  for those who prefer short lowercase versions
#define dbprintf JrDebugPrintf
#define dbprints JrDebugPrintString
#define dbassert JrDebugAssert
#define dbverify JrDebugVerify
#define dbthrow JrDebugThrow
//
#define debugout JrDebugStream
#define dberror JrDebugPrintfError
#define debugerr JrDebugStreamError
//
#define debugif JrDebugIf
//
#define debugblock JrDebugBlock
#define debugblockreport JrDebugBlockReport
#define dbenclose JrDebugEnclose
//
#define debugpush JrDebugActivityPush
#define debugpop JrDebugActivityPop
//
#define dbtest JrDebugTest
#define dbtestex JrDebugTestEx
#define dbtestinfo JrDebugTestInfo
#define dbtestexinfo JrDebugTestExInfo
#define dbtestrun JrDebugTestRun
//
#define JrDebugTrace JrDebugPrintf
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Shortcuts to make it easier for user to do short #ifdefs in their own code
#ifndef JrDebugDIRECTIVE_DontCompileDebugging
 #define JrDebuggingFlag true
 #define JrDebugging
#else
 #define JrDebuggingFlag false
#endif
//---------------------------------------------------------------------------










//---------------------------------------------------------------------------
// Now include the header files depending on whether jrdebug is compiled in or not

#ifdef JrDebugDIRECTIVE_DontCompileDebugging

	//---------------------------------------------------------------------------
	// System includes needed when debug is not compiled in
	//
	//---------------------------------------------------------------------------

	//---------------------------------------------------------------------------
	// User macros that are used when debugging is not compiled in
	#include "jrdebug_notcompiled_macros.h"
	//---------------------------------------------------------------------------

#else

	//---------------------------------------------------------------------------
	// System includes needed when debug is compiled in
	// ATTN: are all these really nesc.?
	#include <stdlib.h>
	#include <iostream>
	#include <cstddef>
	#include <string>
	#include <sstream>
	#include <stdio.h>
	#include <stdarg.h>
	#include <time.h>
	#include <string.h>

	// these are needed for getting working directory and special folder location for guess a safe path to save relative specified debug output file pths
	#ifdef _WIN32JRD
		#include "windows.h"
		#include <direct.h>
		#include <Shlobj.h>
	#else
		#include <unistd.h>
	#endif
	//---------------------------------------------------------------------------

	//---------------------------------------------------------------------------
	// User macros that are used when debugging is compiled in
	#include "jrdebug_compiled_macros.h"
	#include "jrdebug_classes.h"
	// other includes are in jrdebug_main.cpp
	//---------------------------------------------------------------------------

#endif
//---------------------------------------------------------------------------







//---------------------------------------------------------------------------
// end of namespace jrdebug
#ifdef JrDebugDIRECTIVE_UseNameSpaces
 };
 // should the namespace automatically visible by anyone who includes this file?
 #ifdef JrdUseNameSpacesAutoUsing
  using namespace jrdebug;
 #endif
#endif
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// FINALLY, ANY POST HEADER COMPILER-SPECIFIC STUFF
//  useful for popping pragmas
#include "jrdebug_compilerspecifics_post.h"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// header guard
#endif
//---------------------------------------------------------------------------
