//---------------------------------------------------------------------------
// jrdebug_notcompiled_macros.h
// substitutes for main macros that user interacts with, when jrdebug
// code is 'compiled-out' ie not used.
//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
// MACROS
// These should compile to nothingness, or else pass through args,
//  or else compile to functions which can consume the arguments passed when
//  necesary.
//---------------------------------------------------------------------------

// just used as empty expression (see JrDebugErrorIfNotCompiledIn)
#define JrDoNothing

// disabled flag
#define JrDebugEnabled 0
#define JrDebugEnable(REASONSTR)
#define JrDebugDisable(REASONSTR)
#define JrDebugQuietCall(X)
//
// swallow args (the function will never be called because condition is always false)
#define JrDebugPrintf if (1) ; else JrDebugSwallowFormatArgs
#define JrDebugPrintString if (1) ; else JrDebugSwallowFormatArgs
#define JrDebugPrintfError if (1) ; else JrDebugSwallowFormatArgs
//
// ifdebug
#define JrDebugIf(X) if (0) 
//
// replacement throw
#define JrDebugThrow(X,Y) throw X
// stream version, just swallow it
#define JrDebugStream if (1) ; else JrDebugStreamSwallower
#define JrDebugStreamError if (1) ; else JrDebugStreamSwallower
// debug block (just swallow it)
#define JrDebugBlock(Nstr) 
#define JrDebugBlockReport(Nstr) 
#define JrDebugActivityPush(Nstr) 
#define JrDebugActivityPop(Nstr) 
// in case you want to test and confirm that debugging is compiled in
#define JrDebugErrorIfNotCompiledIn JRDEBUG_ERROR_This_error_is_triggered_on_purpose_using_the_ErrorIfNotCompiledIn_macro.__JrDebugLogger_is_being_compiled-out(!)_remove_this_line_or_compile-in_JrDebugLogger
#define JrDebugErrorIfCompiledIn JrDoNothing
// debug enclose a function call - here this just calls the function
#define JrDebugEnclose(X) X
// unit testing
#define JrDebugTest(X)
#define JrDebugTestEx(X)
#define JrDebugTestInfo(X,Y)
#define JrDebugTestExInfo(X,Y)
#define JrDebugRegisterTestFunction(SimpName,Desc,FName,KString)
#define JrDebugRunTestFunction(SimpName,Desc,FName)
#define JrDebugTestRun(FPOINT)
// flags
#define JrDebugEnableExceptionLogging(LEVEL)
#define JrDebugSetDisplayPassingAsserts(BOOLVAL) 
// announce (ATTN: maybe we should print "debug disabled" in this case?)
#define JrDebugAnnounce(APPNAME,BOOLVAL,ARGC,ARGV)
// accessors
#define JrDebugSetDefaultOutputOptions(OPSTRING)
// output formats
#define JrDebugRegisterOutputFormatter(ObjName)
// callback func (we put an impossible execution in here to avoid unused variable warnings
#define JrDebugSetCallbackPointer(CALLBACKFP) { if (0) {JrdUnusedArg(*CALLBACKFP);} }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// assert and verify (note user is responsible for #inlcuding <cassert> if they plan to use these in non debug)
#ifdef ASSERT
  #define JrDebugAssert(X) ASSERT(X)
#else
  #define JrDebugAssert(X) assert(X)
#endif

#ifdef VERIFY
 #define JrDebugVerify(X) VERIFY(X)
#else
 #define JrDebugVerify(X) assert(X)
#endif
//---------------------------------------------------------------------------









//---------------------------------------------------------------------------
// swallow args (these are never actually called due to an impossible if (0)
//  check, but they are necesary for compiler syntax checking.
// touch unused args to avoid compiler warnings
inline void JrDebugSwallowFormatArgs(const char* format, ...) {JrdUnusedArg(format);};
inline void JrDebugSwallowFormatArgs(const jrd_stdstringT &str) {JrdUnusedArg(str);};
inline void JrDebugSwallowFormatArgs(const JrdType inmtype , const char* format, ... ) {JrdUnusedArg(format);JrdUnusedArg(inmtype);};
inline void JrDebugSwallowFormatArgs(const JrdType inmtype , const jrd_stdstringT &str ) {JrdUnusedArg(str);JrdUnusedArg(inmtype);};
inline void JrDebugSwallowFormatArgs(const JrdType inmtype , const int inseverity, const char* format, ... ) {JrdUnusedArg(format);JrdUnusedArg(inmtype);JrdUnusedArg(inseverity);};
inline void JrDebugSwallowFormatArgs(const JrdType inmtype , const int &inseverity, const jrd_stdstringT &str) {JrdUnusedArg(str);JrdUnusedArg(inmtype);JrdUnusedArg(inseverity);};
inline void JrDebugSwallowFormatArgs(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const char* format, ... ) {JrdUnusedArg(format);JrdUnusedArg(dmsubtypestr);JrdUnusedArg(inactivity);JrdUnusedArg(inmtype);JrdUnusedArg(inseverity);};
inline void JrDebugSwallowFormatArgs(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int &inseverity, const jrd_stdstringT &str) {JrdUnusedArg(str);JrdUnusedArg(dmsubtypestr);JrdUnusedArg(inactivity);JrdUnusedArg(inmtype);JrdUnusedArg(inseverity);};
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// for stream swallowing
//  again these functions will NEVER actually get called because their
//  invocation is always preceded by if (0)..., but we must provide them so
//  that syntactically correct stream io is available to swallow << for
//  static compilation errors (thx mauke on #c++@efnet for the template syntax help)
// ATTN: this uses a template; we might want to make a version that doesn't for
//  platforms without good template support; the JrDebugS class derives from stream
//  in order to provide this functionality (but is using stringstream equiv. to using tempaltes?)
class MyNullStreamT
{
public:
	// touch unused args to avoid compiler warnings
	const MyNullStreamT& operator()() {return *this;};
	const MyNullStreamT& operator()(const JrdType inmtype , const int inseverity=0) {JrdUnusedArg(inmtype); JrdUnusedArg(inseverity); return *this;};
    const MyNullStreamT& operator()(const JrdType inmtype , const char *indmsubtypestr, const char *inactivity, const int inseverity) const { JrdUnusedArg(inmtype);JrdUnusedArg(indmsubtypestr);JrdUnusedArg(inactivity);JrdUnusedArg(inseverity); return *this; }
	template<typename T>
	const MyNullStreamT& operator<<(const T &) const { return *this; }
};
// we need a single global object (declared in jrdebug_main.cpp)
extern MyNullStreamT JrDebugStreamSwallower;
//---------------------------------------------------------------------------









//---------------------------------------------------------------------------
// Commandline Parser Function
#define JrDebugParseCommandlineArgc(DESC,ARGCP,ARGV,DEFAULTCOMLINE) JrDebugParseCommandline(DESC,ARGCP,ARGV,"","",DEFAULTCOMLINE,"")
#define JrDebugParseCommandlineStr(DESC,ARGLINE,DEFAULTCOMLINE) JrDebugParseCommandline(DESC,(int*)0,(char**)0,ARGLINE,"",DEFAULTCOMLINE,"")
#define JrDebugParseCommandlineArgString(DESC,ARG0,FOUNDARGS) JrDebugParseCommandline("jrdebug",(int*)0,(char**)0,ARG0,"","","")
//
#ifdef JrDebugDIRECTIVE_CompiledOutCommandlineParseOff
  // just swallow the commandline args
  #define JrDebugParseCommandline(DESC,ARGCP,ARGV,ALTCOMLINESTR,PRECOMLINEOPTS,DEFAULTCOMLINE,DEFAULTOUTPUTOPTIONS) (true ? false : (DESC!=0 && ARGCP!=0 && ARGV==0 && ALTCOMLINESTR!=0 && PRECOMLINEOPTS!=0 && DEFAULTCOMLINE!=0 && DEFAULTOUTPUTOPTIONS!=0))
#else
  // create a minimal commandline parser that just checks for commandline debug args and warns if found that jrdebug was not compiled in
  bool JrDebugParseCommandline(char *inprogramdesc,int *argcp,char **argv, char* altcomlinestr, char *precomline, char* defaultcomline, char *indefaultoutputoptions)
	{
	if (argcp!=NULL)
		{
    	for (int count=1;count<*argc;++count)
			{
			if (strstr(argv[count],"-db")==argv[count]) { puts("ERROR: JrdDebug commandline arguments ignored since JrDebug functionality was not compiled into this program."); return true;};
			}
		}
	if (altcomlinestr!=NULL)
		{
		if strstr(altcomelinestr,"-db")!=NULL) { puts("ERROR: JrdDebug commandline arguments ignored since JrDebug functionality was not compiled into this program."); return true;};
		}
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(inprogramdesc);
	JrdUnusedArg(precomline);
	JrdUnusedArg(defaultcomline);
	JrdUnusedArg(indefaultoutputoptions);
    return false;
    }
#endif
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// dummy callback class so user can still derive from it in their code, even
//  though its no longer used
class JrDebugCallback
{
public:
	virtual bool EventNotify(const char *fulltext)=0;
};
//---------------------------------------------------------------------------
